@@ -15,7 +15,7 @@ from ._emai import _emai
 
 
 def unbalance_varcom(data_file, id, tpoint, trait, agmat_file, tfix=None, fix=None, forder=3, aorder=3, porder=3,
-             na_method='omit', init=None, maxiter=100, cc_par=1.0e-8, cc_gra=1.0e-3,
+             na_method='omit', init=None, maxiter=100, cc_par=1.0e-8, cc_gra=1.0e-3, crank=False,
                      em_weight_step=0.01, out_file='unbalance_varcom'):
     """
     Estimate the variance parameters for the unbalance longitudinal data using the random regression model
@@ -40,6 +40,7 @@ def unbalance_varcom(data_file, id, tpoint, trait, agmat_file, tfix=None, fix=No
     :param maxiter: the maximum number of iteration. Default is 100.
     :param cc_par: Convergence criteria for the changed variance parameters. Default is 1.0e-8.
     :param cc_gra: Convergence criteria for the norm of gradient vector. Default is 1.0e-3.
+    :param crank: force the column full rank of fixed effects. Default is False.
     :param em_weight_step: the step of the em weight. Default is 0.001.
     :param out_file: the prefix for the output file. Default is 'unbalance_varcom'.
     :return: the estimated variance parameters.
@@ -60,7 +61,7 @@ def unbalance_varcom(data_file, id, tpoint, trait, agmat_file, tfix=None, fix=No
     xmat_t = np.concatenate(xmat_t, axis=1)
     if tfix is not None:
         mat = []
-        _, dmat_t, _, _ = _design_matrix(tfix, data_df)
+        dmat_t = _design_matrix(tfix, data_df)
         for i in range(dmat_t.shape[1]):
             mat.append(xmat_t * dmat_t[:, i:(i+1)])
         xmat_t = np.concatenate(mat, axis=1)
@@ -69,8 +70,9 @@ def unbalance_varcom(data_file, id, tpoint, trait, agmat_file, tfix=None, fix=No
     logging.info('Time independent fixed effect: {}'.format(fix))
     xmat = xmat_t
     if fix is not None:
-        _, xmat_nt, _, _ = _design_matrix(fix, data_df)
+        xmat_nt = _design_matrix(fix, data_df)
         xmat = np.concatenate([xmat_t, xmat_nt[:, 1:]], axis=1)
+    if crank:
         _, indexes = sympy.Matrix(xmat).rref()
         xmat = xmat[:, indexes]
     logging.info('***Build the dedign matrix for random effect***')
